@@ -8,7 +8,7 @@
 #
 package List::DoubleLinked;
 BEGIN {
-  $List::DoubleLinked::VERSION = '0.002';
+  $List::DoubleLinked::VERSION = '0.003';
 }
 
 use strict;
@@ -17,12 +17,14 @@ use warnings FATAL => 'all';
 use Carp qw/carp/;
 use Scalar::Util 'weaken';
 use namespace::clean 0.20;
+#no autovivication;
 
 sub new {
 	my ($class, @items) = @_;
 	my $self = bless {
 		head => undef,
 		tail => undef,
+		size => 0,
 	}, $class;
 	$self->push(@items);
 	return $self;
@@ -38,9 +40,10 @@ sub push {
 			prev => $self->{tail},
 			next => undef,
 		};
-		$self->{tail}{next} = $new_tail if $self->{tail};
+		$self->{tail}{next} = $new_tail if defined $self->{tail};
 		$self->{tail}       = $new_tail;
 		$self->{head}       = $new_tail if not defined $self->{head};
+		$self->{size}++;
 	}
 	return;
 }
@@ -50,7 +53,8 @@ sub pop {
 	my $ret  = $self->{tail};
 	return if not defined $ret;
 	$self->{tail} = $ret->{prev};
-	$self->{tail}{next} = undef if $self->{tail};
+	$self->{tail}{next} = undef if defined $self->{tail};
+	$self->{size}--;
 	return $ret->{item};
 }
 
@@ -62,9 +66,10 @@ sub unshift {
 			prev => undef,
 			next => $self->{head},
 		};
-		$self->{head}{prev} = $new_head if $self->{head};
+		$self->{head}{prev} = $new_head if defined $self->{head};
 		$self->{head}       = $new_head;
 		$self->{tail}       = $new_head if not defined $self->{tail};
+		$self->{size}++;
 	}
 	return;
 }
@@ -74,14 +79,15 @@ sub shift {
 	my $ret  = $self->{head};
 	return if not defined $ret;
 	$self->{head} = $ret->{next};
-	$self->{head}{prev} = undef if $self->{tail};
+	$self->{head}{prev} = undef if defined $self->{tail};
+	$self->{size}--;
 	return $ret->{item};
 }
 
 sub flatten {
 	my $self = CORE::shift;
 	my @ret;
-	for (my $current = $self->{head} ; $current ; $current = $current->{next}) {
+	for (my $current = $self->{head} ; defined $current ; $current = $current->{next}) {
 		CORE::push @ret, $current->{item};
 	}
 	return @ret;
@@ -89,26 +95,22 @@ sub flatten {
 
 sub front {
 	my $self = CORE::shift;
-	return $self->{head} ? $self->{head}{item} : undef;
+	return defined $self->{head} ? $self->{head}{item} : undef;
 }
 
 sub back {
 	my $self = CORE::shift;
-	return $self->{tail} ? $self->{tail}{item} : undef;
+	return defined $self->{tail} ? $self->{tail}{item} : undef;
 }
 
 sub empty {
 	my $self = CORE::shift;
-	return not defined $self->{head};
+	return not $self->{size};
 }
 
 sub size {
 	my $self = CORE::shift;
-	my $ret  = 0;
-	for (my $current = $self->{head} ; $current ; $current = $current->{next}) {
-		$ret++;
-	}
-	return $ret;
+	return $self->{size};
 }
 
 sub insert_before {
@@ -120,10 +122,11 @@ sub insert_before {
 			prev => $node->{prev},
 			next => $node,
 		};
-		$node->{prev}{next} = $new_node if $node->{prev};
+		$node->{prev}{next} = $new_node if defined $node->{prev};
 		$node->{prev} = $new_node;
 
-		$self->{head} = $node->{next} if $self->{head} and $self->{head} == $node;
+		$self->{head} = $node->{next} if defined $self->{head} and $self->{head} == $node;
+		$self->{size}++;
 	}
 	return;
 }
@@ -137,11 +140,12 @@ sub insert_after {
 			prev => $node,
 			next => $node->{next},
 		};
-		$node->{next}{prev} = $new_node if $node->{next};
+		$node->{next}{prev} = $new_node if defined $node->{next};
 		$node->{next} = $new_node;
 
-		$self->{tail} = $new_node if $self->{tail} and $self->{tail} == $node;
+		$self->{tail} = $new_node if defined $self->{tail} and $self->{tail} == $node;
 		$node = $new_node;
+		$self->{size}++;
 	}
 	return;
 }
@@ -149,12 +153,13 @@ sub insert_after {
 sub erase {
 	my ($self, $node) = @_;
 
-	$node->{prev}{next} = $node->{next} if $node->{prev};
-	$node->{next}{prev} = $node->{prev} if $node->{next};
+	$node->{prev}{next} = $node->{next} if defined $node->{prev};
+	$node->{next}{prev} = $node->{prev} if defined $node->{next};
 
-	$self->{head} = $node->{next}     if $self->{head} and $self->{head} == $node;
-	$self->{tail} = $node->{previous} if $self->{tail} and $self->{tail} == $node;
+	$self->{head} = $node->{next}     if defined $self->{head} and $self->{head} == $node;
+	$self->{tail} = $node->{previous} if defined $self->{tail} and $self->{tail} == $node;
 
+	$self->{size}--;
 	weaken $node;
 	carp 'Node may be leaking' if $node;
 
@@ -181,7 +186,9 @@ sub DESTROY {
 	while ($current) {
 		delete $current->{prev};
 		$current = delete $current->{next};
+		$self->{size}--;
 	}
+	warn "Size of Linked List is $self->{size}, should be 0 after DESTROY" if $self->{size} != 0;
 	return;
 }
 
@@ -199,7 +206,7 @@ List::DoubleLinked - Double Linked Lists for Perl
 
 =head1 VERSION
 
-version 0.002
+version 0.003
 
 =head1 SYNOPSIS
 
@@ -210,7 +217,7 @@ version 0.002
 
 =head1 DESCRIPTION
 
-This module provides a double linked list for Perl. You should ordinarily use arrays instead of this, they are faster for almost any usage. However there is a small set of use-cases where linked lists are necessary. This module was written in particular to offer stable iterators (iterators that will remain valid even if other elements are added or removed anywhere in the list).
+This module provides a double linked list for Perl. You should ordinarily use arrays instead of this, they are faster for almost any usage. However there is a small set of use-cases where linked lists are necessary. While you can use the list as an object directly, for most purposes it's recommended to use iterators. C<begin()> and C<end()> will give you iterators pointing at the start and end of the list.
 
 =head1 METHODS
 
@@ -218,6 +225,14 @@ This module provides a double linked list for Perl. You should ordinarily use ar
 
 Create a new double linked list. @elements is pushed to the list.
 
+=head2 begin()
+
+Return an L<iterator|List::Double::Linked::Iterator> to the first element of the list.
+
+=head2 end()
+
+Return an L<iterator|List::Double::Linked::Iterator> to the last element of the list.
+
 =head2 flatten()
 
 Return an array containing the same values as the list does. This runs in linear time.
@@ -254,14 +269,6 @@ Returns true if the list has no elements in it, returns false otherwise.
 
 Return the length of the list. This runs in linear time.
 
-=head2 begin()
-
-Return an iterator to the first element of the list.
-
-=head2 end()
-
-Return an iterator to the last element of the list.
-
 =head2 erase($iterator)
 
 Remove the element under $iterator.
@@ -274,6 +281,12 @@ Insert @elements before $iterator.
 
 Insert @elements after $iterator.
 
+=head1 WTF WHERE YOU THINKING?
+
+This module is a bit an exercise in C programming. I was surprised that I was ever going to need this (and even more surprised no one ever uploaded something like this to CPAN before), but I do. B<I need a data structure that provided me with stable iterators>. I need to be able to splice off any arbitrary element without affecting any other arbitrary element. You can't really implement that using arrays, you need a double linked list for that.
+
+This module is optimized for correctness, both algorithmically as memory wise. It is not optimized for speed. Linked lists in Perl are practically never faster than arrays anyways, so if you're looking at this because you think it will be faster think again. L<splice|perlfunc/"splice"> is your friend.
+
 =head1 AUTHOR
 
 Leon Timmermans <fawaka@gmail.com>
